(function (exports) {
	'use strict';

	var lookup = [
		'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
		'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
		'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
		'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
		'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
		'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
		'w', 'x', 'y', 'z', '0', '1', '2', '3',
		'4', '5', '6', '7', '8', '9', '+', '/'
	];

	function uint8ToBase64(uint8) {
		var i,
			extraBytes = uint8.length % 3, // if we have 1 byte left, pad 2 bytes
			output = "",
			temp, length;

		function tripletToBase64 (num) {
			return lookup[num >> 18 & 0x3F] + lookup[num >> 12 & 0x3F] + lookup[num >> 6 & 0x3F] + lookup[num & 0x3F];
		};

		// go through the array every three bytes, we'll deal with trailing stuff later
		for (i = 0, length = uint8.length - extraBytes; i < length; i += 3) {
			temp = (uint8[i] << 16) + (uint8[i + 1] << 8) + (uint8[i + 2]);
			output += tripletToBase64(temp);
		}

		switch (extraBytes) {
			case 1:
				output += 'AA';
				break;
			case 2:
				output += 'A';
				break;
		}

		// this prevents an ERR_INVALID_URL in Chrome (Firefox okay)
		switch (output.length % 4) {
			case 1:
				output += '=';
				break;
			case 2:
				output += '==';
				break;
			default:
			break;
		}

		return output;
	}

	function openDataURL(dataURI, name) {
		window.open(dataURI, name || '_self');
	}

	function createDataURL (data, mimeType, autoDownload) {
		var tmp, base64, dataURI, i;

		if (!data || (typeof data !== 'string' && !(data instanceof Array) && !(data instanceof Uint8Array))) {
			throw 'Data must be a non-empty string, Array, or Uint8Array';
		}

		if (typeof mimeType !== 'string') {
			autoDownload = mimeType;
			mimeType = 'application/octet-stream';
		}
		if (typeof autoDownload !== 'boolean') {
			autoDownload = false;
		}

		if (typeof data === 'string') {
			tmp = encodeURIComponent(data);
			tmp = unescape(tmp || data.toString());
			base64 = btoa(tmp);
		} else if (data instanceof Array) {
			tmp = new Uint8Array(data.length);
			for (i = data.length; i >= 0; i -= 1) {
				tmp[i] = data[i] & 0xFF;
			}
			base64 = uint8ToBase64(tmp);
		} else if (data instanceof Uint8Array) {
			base64 = uint8ToBase64(data);
		}

		dataURI = 'data:' + mimeType + ';base64,' + base64;

		if (autoDownload) {
			openDataURL(dataURI);
		}

		return dataURI;
	}

	exports.toDataURL = createDataURL;
	exports.openDataURL = openDataURL;
}(typeof module !== 'undefined' ? module.exports : window));
